import { loadMapModules, getMapModules } from './loadMapModules';
const _ = require('lodash');
const querystring = require('querystring');
const modules = {
  jsonUtils: 'gis/geometry/support/jsonUtils',
};

const layerOperation = {
  isInGroup(lyr, reorderGroups = []) {
    let { lyrIndex, groupIndex } = this.getGroup(lyr, reorderGroups);
    return _.isNumber(lyrIndex) && groupIndex >= 0;
  },
  getLayerWeight(lyr, reorderGroups) {
    if ('graphics' == lyr.type) {
      let { lyrIndex, groupIndex } = this.getGroup(lyr, reorderGroups);
      if (this.isInGroup(lyr, reorderGroups)) return (groupIndex + 1) * 100 + lyrIndex;
    } else {
      return -1;
    }
  },
  registerReorderStrategy: (groupName, reorderGroups = []) => {
    var index = reorderGroups.indexOf(groupName);
    return index < 0 && reorderGroups.push(groupName);
  },
  getGroup(lyr, reorderGroups) {
    let groupName = lyr.order.split('-')[0],
      lyrIndex = Number(lyr.order.split('-')[1]),
      groupIndex = reorderGroups.indexOf(groupName);
    return { lyrIndex, groupIndex };
  },
};

const urlUtils = {
  urlToObject: (url) => {
    let [path, params] = url.split('?');
    let query = querystring.parse(params);
    return { path, query };
  },
  objectToUrl: (url, query) => {},
};

const geomUtils = {
  toGeometry(jsonObj, view) {
    const { jsonUtils } = getMapModules();
    if (jsonObj) {
      jsonObj.spatialReference = {
        wkid: view.spatialReference.wkid,
      };
      return jsonUtils.fromJSON(jsonObj);
    }
  },
};

const loadUtilMapModules = async () => {
  await loadMapModules(modules);
};

const mapUtils = {
  getNearestScale(extent, view) {
    let resolution = extent.width / view.width,
      zoom = 0,
      nearestScale = -1,
      diff = Number.POSITIVE_INFINITY;
    if (view.map.basemap && view.map.basemap.baseLayers.length > 0) {
      view.map.basemap.baseLayers.getItemAt(0).tileInfo.lods.forEach(
        function (item) {
          if (item.resolution >= resolution) {
            var difftemp = item.resolution - resolution;
            if (difftemp < diff) {
              diff = difftemp;
              nearestScale = item.scale;
              zoom = item.level;
            }
          }
        }.bind(this),
      );
    }
    return { scale: nearestScale, zoom };
  },
};

export { layerOperation, urlUtils, loadUtilMapModules, geomUtils, mapUtils };
